% Copyright (C) 2012  Arno Onken
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

% -------------------------------------------------------------------------
% Demonstration of a Monte Carlo maximum entropy test showing how to apply
% the test to spike count data.
% -------------------------------------------------------------------------

% Number of test repetitions
n_trials = 20;
% Number of samples to draw in each trial
n_samples = 100;
% Alpha level of the test
alpha = 0.05;
% Poisson firing rate
rate = 3; % Corresponds to 30 Hz for 100 ms bins

%% Apply test to independent Poisson samples
display ('Applying test to independent Poisson samples...');
% Rejection flags
h_ind = zeros (n_trials, 1);
% p-values
p_ind = zeros (n_trials, 1);
for i = 1:n_trials
    disp ([' Trial ' int2str(i) ' of ' int2str(n_trials)]);
    % Draw independent Poisson samples
    x = poissrnd (rate * ones (n_samples, 2));
    % Apply test
    [h_ind(i), p_ind(i)] = mcmaxentpoisstest (x(:, 1), x(:, 2), alpha);
end

%% Apply test to samples from a higher-order distribution
display ('Applying test to higher-order samples...');
% Correlation parameters of mixture components
rho1 = 0.9;
rho2 = -0.9;
% Rejection flags
h_ho = zeros (n_trials, 1);
% p-values
p_ho = zeros (n_trials, 1);
for i = 1:n_trials
    disp ([' Trial ' int2str(i) ' of ' int2str(n_trials)]);
    % Draw samples from a higher-order mixture distribution
    u = normcdf (mvnrnd ([0 0], [1 rho1; rho1 1], n_samples), 0, 1);
    u2 = normcdf (mvnrnd ([0 0], [1 rho2; rho2 1], n_samples), 0, 1);
    z = rand (n_samples, 1);
    u(z>0.5, :) = u2(z>0.5, :);
    % Transform uniform marginals to Poisson marginals
    x = poissinv (u, rate);
    % Apply test
    [h_ho(i), p_ho(i)] = mcmaxentpoisstest (x(:, 1), x(:, 2), alpha);
end

%% Display results
disp (['Rejections for independent Poisson samples: ' num2str(mean (h_ind)*100) '%']);
disp (['Rejections for higher-order samples:        ' num2str(mean (h_ho) * 100) '%']);
